/*
    A simple DirectFB RFB client written specifically for the VIP1710 set top box
    Copyright (C) 2010  Hans-Werner Hilse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


    concepts shamelessly stolen from libvncserver (and thus libvncclient)
    (which this depends upon, BTW)
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <rfb/rfbclient.h>
#include <directfb.h>

static IDirectFB *dfb = NULL;
static IDirectFBSurface *primary = NULL;
static int screen_width  = 0;
static int screen_height = 0;

#define DFBCHECK(x...)                                         \
  {                                                            \
    DFBResult err = x;                                         \
                                                               \
    if (err != DFB_OK)                                         \
      {                                                        \
        fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ ); \
        DirectFBErrorFatal( #x, err );                         \
      }                                                        \
  }


static void PrintRect(rfbClient* client, int x, int y, int w, int h) {
	unsigned int *buf;
	int pitch;
#ifndef NO_EMPTY_HACK
	int emptyflag=0;
#endif /* NO_EMPTY_HACK */

	rfbClientLog("Received an update for %d,%d,%d,%d.\n",x,y,w,h);

	/* check Bpp, currently only 32bit */
	if(client->format.bitsPerPixel != 32) {
		rfbClientLog("Bpp is not 32, cowardly refusing to work.\n");
		return;
	}

#ifndef NO_EMPTY_HACK
	/* hack for making image fully transparent if only zeros are
	 * present _and_ the dimensions are 720x576
	 */
	if((w==720)&&(h==576)&&(x==0)&&(y==0)) {
		rfbClientLog("Checking for empty framebuffer...\n");
		emptyflag=1;
	}
#endif /* NO_EMPTY_HACK */

	/* check dimensions */
	if((x > screen_height) || (y > screen_height))
		return;

	w = ((x+w) <= screen_width) ? w : (screen_width - x);
	h = ((y+h) <= screen_height) ? h : (screen_height - y);

	/* lock framebuffer and blit */
	DFBCHECK (primary->Lock(primary, DSLF_WRITE, (void**) &buf, &pitch)); 
	pitch>>=2; /* is pitch/4 */
	int i,j;
	for(j=0; j<h; j++) {
		const unsigned int* lstart_src =
			(unsigned int*)client->frameBuffer
			+ (((j+y)*client->width) + x);
		unsigned int* lstart_dst = buf + (j+y)*pitch + x;
		for(i=0; i<w; i++) {
			const unsigned int l = lstart_src[i];
			/* convert BGR to RGB w/ full 50% alpha */
			lstart_dst[i] = 0x80000000
					| ((l & 0x00ff0000) >> 16)
					| (l & 0x0000ff00)
					| ((l & 0x000000ff) << 16);
#ifndef NO_EMPTY_HACK
			/* check for empty picture if we're still
			 * searching for colored pixels:
			 */
			if(emptyflag && (l & 0x00ffffff))
				emptyflag = 0;
#endif /* NO_EMPTY_HACK */
		}
	}
#ifndef NO_EMPTY_HACK
	if(emptyflag) {
		/* empty picture, set full 100% alpha */
		rfbClientLog("framebuffer emptied, setting fully transparent.\n");
		memset(buf, 0, h*pitch*4);
	}
#endif /* NO_EMPTY_HACK */
	DFBCHECK (primary->Unlock(primary));
/*
	DFBCHECK (primary->Flip(primary, NULL, 0));
*/
}

int
main(int argc, char **argv)
{
	int quit = 0;
	DFBSurfaceDescription dsc;

	DFBCHECK (DirectFBInit (&argc, &argv));
	DFBCHECK (DirectFBCreate (&dfb));
	DFBCHECK (dfb->SetCooperativeLevel (dfb, DFSCL_FULLSCREEN));  

	dsc.flags = DSDESC_CAPS;
	dsc.caps  = DSCAPS_PRIMARY /* | DSCAPS_FLIPPING */;

	DFBCHECK (dfb->CreateSurface(dfb, &dsc, &primary));
	DFBCHECK (primary->Clear(primary, 0, 0, 0, 0));
	DFBCHECK (primary->GetSize(primary, &screen_width, &screen_height));

	rfbClient* client = rfbGetClient(8,3,4);
	time_t t=time(NULL);

	client->GotFrameBufferUpdate = PrintRect;

	if (!rfbInitClient(client,&argc,argv))
		return 1;

	while (!quit) {
		int i=WaitForMessage(client,1000000);
		if(i<0) {
			quit=1;
		} else if(i>0) {
			if(!HandleRFBServerMessage(client))
				quit=1;
		}
	}

	rfbClientCleanup(client);

	primary->Release( primary );
	dfb->Release( dfb );
	return 0;
}


